﻿using Buddy.Coroutines;
using Styx;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.Pathing;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Seeker.Classes
{
    public class Harvesting : Common
    {
        private static WaitTimer Interacttimer = new WaitTimer(TimeSpan.FromSeconds(2));
        private static HashSet<int> _timberids;
        private static HashSet<int> TimberIDs
        {
            get { return _timberids ?? (_timberids = CreateTimberIDs); }
        }
        public static void HarvestCheck()
        {
            if (BotPoi.Current.Type == PoiType.None)
            {
                IEnumerable<WoWGameObject> LootableList = ObjectManager.GetObjectsOfType<WoWGameObject>().Where(o =>
                    IsValid(o) &&
                    (Herb(o) ||
                    Timber(o) ||
                    Mineral(o) ||
                    MineralChest(o) ||
                    Chest(o) ||
                    SupplyCrate(o) ||
                    GarrisonCache(o)) &&
                    Navigator.CanNavigateFully(StyxWoW.Me.Location, o.Location));
                WoWGameObject ToLoot = LootableList.OrderBy(t=> t.Location.Distance(StyxWoW.Me.Location)).FirstOrDefault();
                    if (ToLoot != null && ToLoot.IsValid)
                    {
                        Logging.Write(name + ": Behaviour created to loot  " + ToLoot.SafeName);
                        BotPoi.Current = new BotPoi(ToLoot, PoiType.Harvest);
                        return;
                    }
                }

        }
        public static async Task<bool> DoHarvest(WoWGameObject ToCut)
        {
            using (new PerformanceLogger(name + ": Harvest"))
            {
                if (ToCut != null && ToCut.IsValid)
                {
                    WoWGuid TargetGuid = ToCut.Guid;
                    if (await Harvest(ToCut))
                    {
                        //I have to assume that if we made it this far that looting is done, Remove it.
                        if (Settings.General.Instance.AdvancedLogging) Logging.Write(name + ":" + ToCut.SafeName + " appears to be looted, removing from our list");
                        Blacklist.Add(TargetGuid, BlacklistFlags.All, TimeSpan.FromMinutes(5));
                        BotPoi.Clear("Harvesting Finished");
                    }
                }
                return false;
            }
        }
        private static async Task<bool> Harvest(WoWGameObject Lootable)
        {
            using (new PerformanceTimer(name + ": Harvest"))
            {
                if (Lootable == null)
                    return true;
                if (Lootable != null)
                {
                    if (!Blacklist.Contains(Lootable.Guid, BlacklistFlags.Loot) && await Movement.MoveToTargetRoutine(Lootable))
                    {
                        if (Lootable.Location.Distance(StyxWoW.Me.Location) <= Lootable.InteractRange && !StyxWoW.Me.IsMoving)
                        {
                            Interacttimer.Reset();
                            Lootable.Interact();
                            await Coroutine.Wait(5000, () => TheSeeker.Instance.LootFrameIsOpen);
                            await Coroutine.Wait(5000, () => !TheSeeker.Instance.LootFrameIsOpen);
                            return !IsValid(Lootable) || Lootable.State != WoWGameObjectState.Ready;
                        }
                    }
                }
                return false;
            }
        }
        private static bool Herb(WoWGameObject o)
        {
            if (!Settings.General.Instance.HarvestHerbs)
                return false;
            return o.SubType == WoWGameObjectType.Chest &&
                    o.IsHerb &&
                    o.Locked == false &&
                    o.LockType == WoWLockType.Herbalism &&
                StyxWoW.Me.IsInInstance;

        }
        private static bool Mineral(WoWGameObject o)
            {
            if (!Settings.General.Instance.HarvestMinerals)
                return false;
            return o.SubType == WoWGameObjectType.Chest &&
                    o.IsMineral &&
                    o.Locked == false &&
                    o.LockType == WoWLockType.Mining &&
                StyxWoW.Me.IsInInstance;

        }
        private static bool MineralChest(WoWGameObject o)
            {
            if (!Settings.General.Instance.HarvestMinerals)
                return false;

            return o.SubType == WoWGameObjectType.Chest &&
                    o.LockType == WoWLockType.OpenKneeling &&
                StyxWoW.Me.IsInInstance;

        }
        private static bool Timber(WoWGameObject o)
            {                
            if (!Settings.General.Instance.HarvestTimber)
                return false;
            return o.SubType == WoWGameObjectType.Goober &&
                    o.CanLoot &&
                    o.Locked == false &&
                TimberIDs.Any(t => o.Entry == t);

        }
        private static bool Chest(WoWGameObject o)
            {
            if (!Settings.General.Instance.OpenLootables)
                return false;
            return o.SubType == WoWGameObjectType.Chest && o.LockType == WoWLockType.Open &&
                o.CanLoot;

        }
        private static bool SupplyCrate(WoWGameObject o)
            {
            if (!Settings.General.Instance.GarrisonSupplies)
                return false;
            return o.SubType == WoWGameObjectType.GarrisonShipment &&
            o.DisplayId == 16091;

        }
        private static bool GarrisonCache(WoWGameObject o)
            {
            if (!Settings.General.Instance.GarrisonSupplies)
                return false;
            return o.SubType == WoWGameObjectType.Goober &&
                    o.Entry == 236916 &&
                o.State == WoWGameObjectState.Ready;

        }

        private static HashSet<int> SmallTimber = new HashSet<int>()
        {
            234109,
            234110,
            233922,
            234097,
            234126,
            234193,
            234197,
            237727,
            234021,
            234080,
            234122,
            233604
        };
        private static HashSet<int> MediumTimber = new HashSet<int>()
        {
            234198,
            234000,
            234123,
            234022,
            234119,
            234196,
            234194,
            234111,
            234098,
            233634,
            234127 
        };
        private static HashSet<int> LargeTimber = new HashSet<int>()
        {
            234023,
            234120,
            234128,
            234199,
            234007,
            234195,
            234124,
            234099,
            233635
        };
        private static HashSet<int> CreateTimberIDs
        {
            get
            {
                HashSet<int> IDList = new HashSet<int>();
                if (lumbermillskill == 1)
                {
                    IDList.UnionWith(SmallTimber);
                }
                if (lumbermillskill == 2)
                {
                    IDList.UnionWith(SmallTimber);
                    IDList.UnionWith(MediumTimber);
                }
                if (lumbermillskill == 3)
                {
                    IDList.UnionWith(SmallTimber);
                    IDList.UnionWith(MediumTimber);
                    IDList.UnionWith(LargeTimber);
                }
                return IDList;
            }
        }
    }
}
