﻿using Buddy.Coroutines;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Styx.CommonBot.Coroutines;

namespace Seeker.Classes
{
    public class Killing : Common
    {
        public static void RareCheck()
        {
            if (BotPoi.Current.Type == PoiType.None)
            {
                WoWUnit ToKill;
                if (Settings.General.Instance.KillRaresonSight)
                {
                    ToKill = Rare;
                    if (IsValid(ToKill))
                    {
                        Logging.Write(name + ": Behaviour created to kill " + ToKill.SafeName + "@" + Math.Round(ToKill.Distance) + "m");
                        BotPoi.Current = new BotPoi(ToKill, PoiType.Kill);
                    }
                }
                if (Settings.General.Instance.CreatureEntries.Length != 0)
                {
                    ToKill = RarebyID(Settings.General.Instance.CreatureEntries);
                    if (IsValid(ToKill))
                    {
                        Logging.Write(name + ": Behaviour created to kill " + ToKill.SafeName);
                        BotPoi.Current = new BotPoi(ToKill, PoiType.Kill);
                    }
                }
            }
        }
        public static async Task<bool> DoKill(WoWUnit ToKill)
        {
            using (new PerformanceLogger(name + ": Kill Rare"))
            {
                if (IsValid(ToKill) && ToKill.IsAlive)
                {
                    WoWGuid TargetGuid = ToKill.Guid;
                    if (await Kill(ToKill))
                    {
                        //I have to assume that if we made it this far that my target is dead, Remove it.
                        if (Settings.General.Instance.AdvancedLogging) Logging.Write(name + ":" + ToKill.SafeName + " appears to be dead, removing from our list");
                        Blacklist.Add(TargetGuid, BlacklistFlags.Combat, TimeSpan.FromMinutes(5));
                        BotPoi.Clear();
                    }
                }
                return false;
            }
        }
        public static async Task<bool> Kill(WoWUnit Rare)
        {
            using (new PerformanceTimer(name + ": Kill"))
            {
                if (Rare == null)
                    return true;
                if (Rare != null)
                {
                    if (!Blacklist.Contains(Rare.Guid, BlacklistFlags.Pull) && await Movement.MoveToTargetRoutine(Rare))
                    {
                        if (StyxWoW.Me.Location.Distance(Rare.Location) < 15) //Don't do anything until we are close enough to hit him
                        {
                            if (StyxWoW.Me.Mounted &&
                                (IsSpellUsable(Settings.General.Instance.SlowFallSpell) ||
                                StyxWoW.Me.BagItems.FirstOrDefault(x => x.Name == name && x.Usable && x.Cooldown <= 0) != null))
                            {
                                Dismount("Need to kill something");
                                await Coroutine.Wait(3000, () => SlowFall());
                            }                            
                            if (StyxWoW.Me.CurrentTarget == null || StyxWoW.Me.CurrentTargetGuid != Rare.Guid) Rare.Target();
                            await TheSeeker.Instance.PullBehavior.ExecuteCoroutine();
                        }
                    }
                    return StyxWoW.Me.Combat;
                }
                return false;
            }
        }
        static public WoWUnit Rare
        {
            get
            {
                return ObjectManager.GetObjectsOfType<WoWUnit>().Where(o => IsValid(o)
                  && o.IsAlive && o.CanSelect && o.Attackable
                  && (o.Classification == WoWUnitClassificationType.Rare)
                  && Navigator.CanNavigateFully(StyxWoW.Me.Location, o.Location)).FirstOrDefault();
            }
        }
        static public WoWUnit RarebyID(int[] entrys)
        {
            return ObjectManager.GetObjectsOfType<WoWUnit>().Where(o => IsValid(o)
                && o.IsAlive && o.CanSelect && o.Attackable
                && entrys.Any(e => o.Entry == e)).FirstOrDefault();

        }
        public static bool SlowFall()
        {
            if (IsSpellUsable(Settings.General.Instance.SlowFallSpell))
            {
                SpellManager.Cast(Settings.General.Instance.SlowFallSpell);
                return true;
            }
            return UseContainerItem(Settings.General.Instance.SlowFallItem);

        }
        public static bool IsSpellUsable(string spell)
        {
            return SpellManager.CanCast(spell);
        }
        public static bool UseContainerItem(string name)
        {
            using (new PerformanceLogger("Use ContainerItem"))
            {
                WoWItem Item = StyxWoW.Me.BagItems.FirstOrDefault(x => x.Name == name && x.Usable && x.Cooldown <= 0);
                if (Item != null && Item.CooldownTimeLeft.TotalMilliseconds == 0)
                {
                    Item.UseContainerItem();
                    return true;
                }
                return false;
            }
        }
        public static void UseContainerItem(string name, Func<bool> req)
        {
            if (req()) UseContainerItem(name);
        }
    }
}
