﻿//=================================================================
//
//				      The Seeker - Plugin
//					  Author: gonemental
//                    
//  Credit goes to HighVolts for the framework of this bot since most of the
//  Framework was taken from AutoAngler2.
//==================================================================

// -- ToDo --

using Buddy.Coroutines;
using Seeker.ConfigWindow;
using Seeker.Classes;
using Styx;
using Styx.Common;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Media;
using System.Linq;
using System.Collections.Generic;
using System.Collections;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.POI;
using Bots.Professionbuddy.BehaviorTree;
using CommonBehaviors.Actions;
using Styx.TreeSharp;
using Styx.Pathing;
using Bots.Grind;
using Styx.CommonBot.Routines;
using System.IO;
using Styx.CommonBot.Profiles;


namespace Seeker
{
    public enum PathingType
    {
        Circle,
        Bounce
    }
    public class TheSeeker : BotBase
    {
        internal bool LootFrameIsOpen { get; private set; }
        private PathingType _pathingType = PathingType.Circle;
        internal TSProfile Profile { get; private set; }
        internal static TheSeeker Instance { get; private set; }
        private static Styx.TreeSharp.Composite _combatBehavior;
        private static Styx.TreeSharp.Composite _restBehavior;
        private static Styx.TreeSharp.Composite _pullBehavior;
        private static Styx.TreeSharp.Composite _deathBehavior;
        private static Styx.TreeSharp.Composite _lootBehavior;

        static Styx.TreeSharp.Composite CombatBehavior
        {
            get { return _combatBehavior ?? (_combatBehavior = RoutineManager.Current.CombatBehavior); }
        }
        static Styx.TreeSharp.Composite RestBehavior
        {
            get { return _restBehavior ?? (_restBehavior = RoutineManager.Current.RestBehavior); }
        }
        public Styx.TreeSharp.Composite PullBehavior
        {
            get { return _pullBehavior ?? (_pullBehavior = RoutineManager.Current.PullBehavior); }
        }
        static Styx.TreeSharp.Composite LootBehavior
        {
            get { return _lootBehavior ?? (_lootBehavior = LevelBot.CreateLootBehavior()); }
        }
        static Styx.TreeSharp.Composite DeathBehavior
        {
            get { return _deathBehavior ?? (_deathBehavior = LevelBot.CreateDeathBehavior()); }
        }
        public static string name { get { return "The Seeker"; } }
        public override bool IsPrimaryType
        {
            get { return true; }
        }
        public override string Name { get { return name; } }
        private readonly static Version _version = new Version(1, 0, 4);
        public override Styx.TreeSharp.Composite Root
        {
            get { return _root ?? (_root = new ActionRunCoroutine(ctx => RootLogic())); }
        }
        private Styx.TreeSharp.Composite _root;
        public static LocalPlayer Me = StyxWoW.Me;
        public TheSeeker()
        {
            Instance = this;
            BotEvents.Profile.OnNewOuterProfileLoaded += Profile_OnNewOuterProfileLoaded;            
        }
        private Form MyForm;
        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All & (~PulseFlags.CharacterManager); }
        }
        
        public override Form ConfigurationForm
        {
            get
            {
                return MyForm ?? (MyForm = new TheSeekerUI());
            }
        }
        public override void Initialize()
        {
            if (File.Exists(Settings.General.Instance.LastLoadedProfile))
                ProfileManager.LoadNew(Settings.General.Instance.LastLoadedProfile);
            else
                ProfileManager.LoadEmpty();
            Logging.Write(Colors.Blue, name +"v"+ _version + " Started");
        }
        //Priority Garrison Supplies, Herbs, Minerals, Timber
        private async Task RootLogic()
        {
            using (new PerformanceLogger(name + ": Root"))
            {
                // Is bot dead? if so, release and run back to corpse
                if (await DeathBehavior.ExecuteCoroutine())
                    return;
                if (await RestBehavior.ExecuteCoroutine())
                    return;
                // Run Combat if needed
                if (Me.Combat && CombatBehavior != null && !Flightor.MountHelper.Mounted)
                    await CombatBehavior.ExecuteCoroutine();
                //NO need to do anything if we are dead, in combat, or resting
                if (!StyxWoW.Me.IsAlive || StyxWoW.Me.Combat || RoutineManager.Current.NeedRest)
                    return;
                // Loot if needed
                if (!StyxWoW.Me.IsFlying && await LootBehavior.ExecuteCoroutine())                
                    return;                
                #region SpecialChecks
                Killing.RareCheck();
                Looting.LootCheck();
                Harvesting.HarvestCheck();
                #endregion
                #region HandlePoi                
                if (BotPoi.Current.Type == PoiType.Kill)
                {
                    var ToKill = BotPoi.Current.AsObject as WoWUnit;
                    if (ToKill == null || !ToKill.IsValid)
                        BotPoi.Clear("Target not found");
                    if (ToKill.IsDead)
                        BotPoi.Clear("Target is dead");
                    await Killing.DoKill(ToKill);
                    return;
                }
                if (BotPoi.Current.Type == PoiType.Loot)
                {
                    var ToLoot = BotPoi.Current.AsObject as WoWUnit;
                    if (ToLoot == null || !ToLoot.IsValid)
                        BotPoi.Clear();
                    await Looting.DoLoot(ToLoot);
                    return;
                }
                if (BotPoi.Current.Type == PoiType.Harvest)
                {
                    var ToHarvest = BotPoi.Current.AsObject as WoWGameObject;
                    if (ToHarvest == null || !ToHarvest.IsValid)
                        BotPoi.Clear();
                    await Harvesting.DoHarvest(ToHarvest);
                    return;
                }
                #endregion
                await Movement.FollowPath();
            }
        }
        public override void Stop()
        {
            Lua.Events.DetachEvent("LOOT_OPENED", LootFrameOpenedHandler);
            Lua.Events.DetachEvent("LOOT_CLOSED", LootFrameClosedHandler);
        }
        public override void Start()
        {
            Lua.Events.AttachEvent("LOOT_OPENED", LootFrameOpenedHandler);
            Lua.Events.AttachEvent("LOOT_CLOSED", LootFrameClosedHandler);            
        }
        public override void Pulse()
        {
            base.Pulse();
        }
        private void LootFrameClosedHandler(object sender, LuaEventArgs args)
        {
            LootFrameIsOpen = false;
        }

        private void LootFrameOpenedHandler(object sender, LuaEventArgs args)
        {
            LootFrameIsOpen = true;
        }
        private void Profile_OnNewOuterProfileLoaded(BotEvents.Profile.NewProfileLoadedEventArgs args)
        {
            try
            {
                Profile = new TSProfile(args.NewProfile, _pathingType);
                if (!string.IsNullOrEmpty(ProfileManager.XmlLocation))
                {
                    Settings.General.Instance.LastLoadedProfile = ProfileManager.XmlLocation;
                    Logging.Write("Loaded:" + Settings.General.Instance.LastLoadedProfile);
                    Settings.General.Instance.Save();
                }
            }
            catch (Exception ex)
            {
                Logging.WriteException(ex);
            }
        }
    }
}
